import pprint


class Error(Exception):
    def print_error(self) -> None:
        pass

    def __str__(self) -> str:
        pp = pprint.PrettyPrinter(indent=4)
        return pp.pformat(vars(self))


from .file_bumper import bump_files  # noqa



import abc


class Action(metaclass=abc.ABCMeta):
    def __init__(self) -> None:
        pass

    @abc.abstractmethod
    def print_self(self) -> None:
        pass

    @abc.abstractmethod
    def do(self) -> None:
        pass



""" Fake hook used for tetsting.
Just write a file named after-hook.stamp when called,
so that test code can check if the hook ran
"""

from path import Path


def main() -> None:
    Path("after-hook.stamp").write_text("")


if __name__ == "__main__":
    main()



import re
from typing import cast, Any, Dict, List, Optional, Pattern  # noqa

import attr
import schema
from path import Path
import tomlkit

from .hooks import HOOKS_CLASSES, Hook, BeforeCommitHook, AfterPushHook  # noqa


@attr.s
class Config:
    current_version = attr.ib()  # type: str
    version_regex = attr.ib()  # type: Pattern[str]

    git_tag_template = attr.ib()  # type: str
    git_message_template = attr.ib()  # type: str

    files = attr.ib()  # type: List[File]
    hooks = attr.ib()  # type: List[Hook]

    github_url = attr.ib()  # type: Optional[str]


@attr.s
class File:
    src = attr.ib()  # type: str
    search = attr.ib(default=None)  # type: Optional[str]
    version_template = attr.ib(default=None)  # type: Optional[str]


def validate_template(name: str, pattern: str, value: str) -> None:
    if pattern not in value:
        message = "%s should contain the string %s" % (name, pattern)
        raise schema.SchemaError(message)


def validate_git_tag_template(value: str) -> None:
    validate_template("git.tag_template", "{new_version}", value)


def validate_git_message_template(value: str) -> None:
    validate_template("git.message_template", "{new_version}", value)


def validate_version_template(
    src: str, version_template: str, known_groups: Dict[str, str]
) -> None:
    try:
        version_template.format(**known_groups)
    except KeyError as e:
        message = "version template for '%s' contains unknown group: %s" % (src, e)
        raise schema.SchemaError(message)


def validate_hook_cmd(cmd: str) -> None:
    try:
        cmd.format(new_version="dummy", current_version="dummy")
    except KeyError as e:
        message = "hook cmd: '%s' uses unknown placeholder: %s" % (cmd, e)
        raise schema.SchemaError(message)


def validate_basic_schema(config: Dict[str, Any]) -> Config:
    """ First pass of validation, using schema """
    # Note: asserts that we won't get KeyError or invalid types
    # when building or initial Config instance
    file_schema = schema.Schema(
        {
            "src": str,
            schema.Optional("search"): str,
            schema.Optional("version_template"): str,
        }
    )

    hook_schema = schema.Schema({"name": str, "cmd": str})

    def validate_re(regex: str) -> str:
        re.compile(regex, re.VERBOSE)
        return regex

    tbump_schema = schema.Schema(
        {
            "version": {"current": str, "regex": schema.Use(validate_re)},
            "git": {"message_template": str, "tag_template": str},
            "file": [file_schema],
            schema.Optional("hook"): [hook_schema],  # retro-compat
            schema.Optional("before_push"): [hook_schema],  # retro-compat
            schema.Optional("before_commit"): [hook_schema],
            schema.Optional("after_push"): [hook_schema],
            schema.Optional("github_url"): str,
        }
    )
    return cast(Config, tbump_schema.validate(config))


def validate_config(cfg: Config) -> None:
    """ Second pass of validation, using the Config
    class.

    """
    # Note: separated from validate_basic_schema to keep error
    # messages user friendly

    current_version = cfg.current_version

    validate_git_message_template(cfg.git_message_template)
    validate_git_tag_template(cfg.git_tag_template)

    match = cfg.version_regex.fullmatch(current_version)
    if not match:
        message = "Current version: %s does not match version regex" % current_version
        raise schema.SchemaError(message)
    current_version_regex_groups = match.groupdict()

    for file_config in cfg.files:
        version_template = file_config.version_template
        if version_template:
            validate_version_template(
                file_config.src, version_template, current_version_regex_groups
            )

    for hook in cfg.hooks:
        validate_hook_cmd(hook.cmd)


def parse(cfg_path: Path) -> Config:
    parsed = tomlkit.loads(cfg_path.text())
    parsed = validate_basic_schema(parsed)
    current_version = parsed["version"]["current"]
    git_message_template = parsed["git"]["message_template"]
    git_tag_template = parsed["git"]["tag_template"]
    version_regex = re.compile(parsed["version"]["regex"], re.VERBOSE)
    files = []
    for file_dict in parsed["file"]:
        file_config = File(
            src=file_dict["src"],
            search=file_dict.get("search"),
            version_template=file_dict.get("version_template"),
        )
        files.append(file_config)

    hooks = []
    for hook_type in ("hook", "before_push", "before_commit", "after_push"):
        cls = HOOKS_CLASSES[hook_type]
        if hook_type in parsed:
            for hook_dict in parsed[hook_type]:
                hook = cls(hook_dict["name"], hook_dict["cmd"])
                hooks.append(hook)

    github_url = parsed.get("github_url")

    config = Config(
        current_version=current_version,
        version_regex=version_regex,
        git_message_template=git_message_template,
        git_tag_template=git_tag_template,
        files=files,
        hooks=hooks,
        github_url=github_url,
    )

    validate_config(config)

    return config



from path import Path
import sys
import tomlkit
import pytest

import tbump.git
import tbump.hooks
import tbump.main


def add_hook(test_repo: Path, name: str, cmd: str, after_push: bool = False) -> None:
    """ Patch the configuration file so that we can also test hooks.

    """
    cfg_path = test_repo / "tbump.toml"
    parsed = tomlkit.loads(cfg_path.text())
    if after_push:
        key = "after_push"
    else:
        key = "before_commit"
    if key not in parsed:
        parsed[key] = tomlkit.aot()
    hook_config = tomlkit.table()
    hook_config.add("cmd", cmd)
    hook_config.add("name", name)
    parsed[key].append(hook_config)

    cfg_path.write_text(tomlkit.dumps(parsed))
    tbump.git.run_git(test_repo, "add", ".")
    tbump.git.run_git(test_repo, "commit", "--message", "update hooks")


def add_before_hook(test_repo: Path) -> None:
    """ Patch config to add a working `before_commit` hook
    that runs tbump/test/data/before.py

    """
    add_hook(
        test_repo,
        "fake yarn",
        sys.executable + " before.py {current_version} {new_version}",
    )


def add_after_hook(test_repo: Path) -> None:
    """ Patch config to add a working `after_push` hook
    that runs tbump/test/data/after.py

    """
    add_hook(test_repo, "after hook", sys.executable + " after.py", after_push=True)


def add_crashing_hook(test_repo: Path) -> None:
    """ Patch config to add a `before_commit` hook
    that runs a command that fails
    """
    add_hook(test_repo, "crashing hook", sys.executable + " nosuchfile.py")


def test_working_hook(test_repo: Path) -> None:
    """
    Check that the configured hook runs and properly uses
    current and new version
    """
    add_before_hook(test_repo)
    tbump.main.main(["-C", test_repo, "1.2.41-alpha-2", "--non-interactive"])
    hook_stamp = test_repo / "before-hook.stamp"
    assert hook_stamp.text() == "1.2.41-alpha-1 -> 1.2.41-alpha-2"


def test_hook_fails(test_repo: Path) -> None:
    """
    Check that the proper exception is raised
    if the hooks exits with non-zero return code
    """
    add_before_hook(test_repo)
    add_crashing_hook(test_repo)
    with pytest.raises(tbump.hooks.HookError):
        tbump.main.run(["-C", test_repo, "1.2.41-alpha-2", "--non-interactive"])


def test_hooks_after_push(test_repo: Path) -> None:
    """
    Check that both `before_commit` and `after_push`
    hooks run when tbump is configured with both
    """
    add_before_hook(test_repo)
    add_after_hook(test_repo)
    tbump.main.main(["-C", test_repo, "1.2.41-alpha-2", "--non-interactive"])
    assert (test_repo / "before-hook.stamp").exists()
    assert (test_repo / "after-hook.stamp").exists()



from path import Path

import tbump.config
import tbump.git
import tbump.git_bumper
from tbump.git_bumper import GitBumper

import pytest


@pytest.fixture
def test_git_bumper(test_repo: Path) -> GitBumper:
    config = tbump.config.parse(test_repo / "tbump.toml")
    git_bumper = tbump.git_bumper.GitBumper(test_repo)
    git_bumper.set_config(config)
    return git_bumper


def test_git_bumper_happy_path(test_repo: Path, test_git_bumper: GitBumper) -> None:
    new_version = "1.2.42"
    test_git_bumper.check_dirty()
    test_git_bumper.check_branch_state(new_version)
    # Make sure git add does not fail:
    # we could use file_bumper here instead
    (test_repo / "VERSION").write_text(new_version)
    commands = test_git_bumper.get_commands(new_version)
    for command in commands:
        command.run()
    _, out = tbump.git.run_git_captured(test_repo, "log", "--oneline")
    assert "Bump to %s" % new_version in out


def test_git_bumper_no_tracking_ref(
    test_repo: Path, test_git_bumper: GitBumper
) -> None:
    tbump.git.run_git(test_repo, "checkout", "-b", "devel")

    with pytest.raises(tbump.git_bumper.NoTrackedBranch):
        test_git_bumper.check_dirty()
        test_git_bumper.check_branch_state("1.2.42")


def test_not_on_any_branch(test_repo: Path, test_git_bumper: GitBumper) -> None:
    tbump.git.run_git(test_repo, "commit", "--message", "test", "--allow-empty")
    tbump.git.run_git(test_repo, "checkout", "HEAD~1")

    with pytest.raises(tbump.git_bumper.NotOnAnyBranch):
        test_git_bumper.check_dirty()
        test_git_bumper.check_branch_state("1.2.42")



from typing import Dict, List, Optional, Pattern
import attr
import re
import glob

from path import Path
import cli_ui as ui

import tbump
import tbump.action
import tbump.git
import tbump.config


@attr.s
class ChangeRequest:
    src = attr.ib()  # type: str
    old_string = attr.ib()  # type: str
    new_string = attr.ib()  # type: str
    search = attr.ib(default=None)  # type: Optional[str]


class Patch(tbump.action.Action):
    # pylint: disable=too-many-arguments
    def __init__(
        self, working_path: Path, src: str, lineno: int, old_line: str, new_line: str
    ):
        super().__init__()
        self.working_path = working_path
        self.src = src
        self.lineno = lineno
        self.old_line = old_line
        self.new_line = new_line

    def print_self(self) -> None:
        # fmt: off
        ui.info(
            ui.red, "- ", ui.reset,
            ui.bold, self.src, ":", ui.reset,
            ui.darkgray, self.lineno + 1, ui.reset,
            " ", ui.red, self.old_line.strip(),
            sep="",
        )
        ui.info(
            ui.green, "+ ", ui.reset,
            ui.bold, self.src, ":", ui.reset,
            ui.darkgray, self.lineno + 1, ui.reset,
            " ", ui.green, self.new_line.strip(),
            sep="",
        )
        # fmt: on

    def do(self) -> None:
        self.apply()

    @staticmethod
    def get_ending(line: bytes) -> bytes:
        if line.endswith(b"\r\n"):
            return b"\r\n"
        else:
            return b"\n"

    def apply(self) -> None:
        file_path = self.working_path / self.src
        contents = file_path.bytes()
        lines = contents.splitlines(keepends=True)
        old_line = lines[self.lineno]
        lines[self.lineno] = self.new_line.encode() + Patch.get_ending(old_line)
        file_path.write_lines(lines, linesep=None)


class BadSubstitution(tbump.Error):
    def __init__(
        self,
        *,
        src: str,
        verb: str,
        groups: Dict[str, str],
        template: str,
        version: str
    ):
        super().__init__()
        self.src = src
        self.verb = verb
        self.groups = groups
        self.template = template
        self.version = version

    def print_error(self) -> None:
        message = [
            " ",
            self.src + ":",
            " refusing to ",
            self.verb,
            " version containing 'None'\n",
        ]
        message += [
            "More info:\n",
            " * version groups:  ",
            repr(self.groups),
            "\n" " * template:        ",
            self.template,
            "\n",
            " * version:         ",
            self.version,
            "\n",
        ]
        ui.error(*message, end="", sep="")


class InvalidVersion(tbump.Error):
    def __init__(self, *, version: str, regex: Pattern[str]):
        super().__init__()
        self.version = version
        self.regex = regex

    def print_error(self) -> None:
        ui.error("Could not parse", self.version, "as a valid version string")


class SourceFileNotFound(tbump.Error):
    def __init__(self, *, src: str):
        super().__init__()
        self.src = src

    def print_error(self) -> None:
        ui.error(self.src, "does not exist")


class CurrentVersionNotFound(tbump.Error):
    def __init__(self, *, src: str, current_version_string: str):
        super().__init__()
        self.src = src
        self.current_version_string = current_version_string

    # TODO: raise just once for all errors
    def print_error(self) -> None:
        ui.error(
            "Current version string: (%s)" % self.current_version_string,
            "not found in",
            self.src,
        )


def should_replace(line: str, old_string: str, search: Optional[str] = None) -> bool:
    if not search:
        return old_string in line
    else:
        return (old_string in line) and (re.search(search, line) is not None)


def on_version_containing_none(
    src: str, verb: str, version: str, *, groups: Dict[str, str], template: str
) -> None:
    raise BadSubstitution(
        src=src, verb=verb, version=version, groups=groups, template=template
    )


class FileBumper:
    def __init__(self, working_path: Path):
        self.working_path = working_path
        self.files = []  # type: List[tbump.config.File]
        self.version_regex = re.compile(".")
        self.current_version = ""
        self.current_groups = {}  # type: Dict[str, str]
        self.new_version = ""
        self.new_groups = {}  # type: Dict[str, str]

    def parse_version(self, version: str) -> Dict[str, str]:
        assert self.version_regex
        match = self.version_regex.fullmatch(version)
        if match is None:
            raise InvalidVersion(version=version, regex=self.version_regex)
        return match.groupdict()

    def set_config(self, config: tbump.config.Config) -> None:
        self.files = config.files
        self.check_files_exist()
        self.version_regex = config.version_regex
        self.current_version = config.current_version
        self.current_groups = self.parse_version(self.current_version)

    def check_files_exist(self) -> None:
        assert self.files
        for file in self.files:
            expected_path = self.working_path / file.src
            files_found = glob.glob(expected_path, recursive=True)
            if not files_found:
                raise SourceFileNotFound(src=file.src)

    def get_patches(self, new_version: str) -> List[Patch]:
        self.new_version = new_version
        self.new_groups = self.parse_version(self.new_version)
        change_requests = self.compute_change_requests()
        tbump_toml_change = ChangeRequest(
            "tbump.toml", self.current_version, new_version
        )
        change_requests.append(tbump_toml_change)
        patches = []
        for change_request in change_requests:
            patches_for_request = self.compute_patches_for_change_request(
                change_request
            )
            patches.extend(patches_for_request)
        return patches

    def compute_patches_for_change_request(
        self, change_request: ChangeRequest
    ) -> List[Patch]:
        old_string = change_request.old_string
        new_string = change_request.new_string
        search = change_request.search
        patches = []

        file_path_glob = self.working_path / change_request.src
        for file_path_str in glob.glob(file_path_glob, recursive=True):
            file_path = Path(file_path_str)
            expanded_src = file_path.relpath(self.working_path)
            old_lines = file_path.lines(retain=False)

            for i, old_line in enumerate(old_lines):
                if should_replace(old_line, old_string, search):
                    new_line = old_line.replace(old_string, new_string)
                    patch = Patch(
                        self.working_path, expanded_src, i, old_line, new_line
                    )
                    patches.append(patch)
        if not patches:
            raise CurrentVersionNotFound(
                src=change_request.src, current_version_string=old_string
            )
        return patches

    def compute_change_requests(self) -> List[ChangeRequest]:
        change_requests = []
        for file in self.files:
            change_request = self.compute_change_request_for_file(file)
            change_requests.append(change_request)
        return change_requests

    def compute_change_request_for_file(self, file: tbump.config.File) -> ChangeRequest:
        if file.version_template:
            current_version = file.version_template.format(**self.current_groups)
            if "None" in current_version:
                on_version_containing_none(
                    file.src,
                    "look for",
                    current_version,
                    groups=self.current_groups,
                    template=file.version_template,
                )
            new_version = file.version_template.format(**self.new_groups)
            if "None" in new_version:
                on_version_containing_none(
                    file.src,
                    "replace by",
                    new_version,
                    groups=self.new_groups,
                    template=file.version_template,
                )
        else:
            current_version = self.current_version
            new_version = self.new_version

        to_search = None
        if file.search:
            to_search = file.search.format(current_version=re.escape(current_version))

        return ChangeRequest(file.src, current_version, new_version, search=to_search)


def bump_files(new_version: str, repo_path: Path = None) -> None:
    repo_path = repo_path or Path(".")
    bumper = FileBumper(repo_path)
    cfg = tbump.config.parse(repo_path / "tbump.toml")
    bumper.set_config(cfg)
    patches = bumper.get_patches(new_version=new_version)
    n = len(patches)
    for i, patch in enumerate(patches):
        ui.info_count(i, n, patch.src)
        patch.print_self()
        patch.apply()



""" Fake hook used for tetsting.
Just write a file named before-hook.stamp when called,
so that test code can check if the hook ran
"""

import sys
from path import Path


def main() -> None:
    current, new = sys.argv[1:]
    Path("before-hook.stamp").write_text(current + " -> " + new)


if __name__ == "__main__":
    main()



from path import Path

import tbump
from tbump.test.conftest import file_contains


def test_bump_files_defaults_to_working_dir(test_repo: Path) -> None:
    with test_repo:
        tbump.bump_files("1.2.42")

    assert file_contains(test_repo / "package.json", '"version": "1.2.42"')


def test_bump_files_with_repo_path(test_repo: Path) -> None:
    tbump.bump_files("1.2.42", test_repo)

    assert file_contains(test_repo / "package.json", '"version": "1.2.42"')



from typing import List, Tuple
from path import Path
import cli_ui as ui

import tbump.action
from tbump.config import Config
import tbump.git


class DirtyRepository(tbump.git.GitError):
    def __init__(self, *, git_status_output: str):
        super().__init__()
        self.git_status_output = git_status_output

    def print_error(self) -> None:
        ui.error("Repository is dirty")
        ui.info(self.git_status_output)


class NotOnAnyBranch(tbump.git.GitError):
    def print_error(self) -> None:
        ui.error("Not on any branch")


class NoTrackedBranch(tbump.git.GitError):
    def __init__(self, *, branch: str):
        super().__init__()
        self.branch = branch

    def print_error(self) -> None:
        ui.error(
            "Current branch (%s)" % self.branch, "does not track anything. Cannot push."
        )


class RefAlreadyExists(tbump.git.GitError):
    def __init__(self, *, ref: str):
        super().__init__()
        self.ref = ref

    def print_error(self) -> None:
        ui.error("git ref", self.ref, "already exists")


class Command(tbump.action.Action):
    def __init__(self, repo_path: Path, cmd: List[str]):
        super().__init__()
        self.repo_path = repo_path
        self.cmd = list(cmd)
        self.verbose = True

    def print_self(self) -> None:
        tbump.git.print_git_command(self.cmd)

    def do(self) -> None:
        self.run()

    def run(self) -> None:
        full_args = [self.repo_path] + self.cmd
        return tbump.git.run_git(*full_args, verbose=False)


class GitBumper:
    def __init__(self, repo_path: Path):
        self.repo_path = repo_path
        self.tag_template = ""
        self.message_template = ""
        self.remote_name = ""
        self.remote_branch = ""
        self.commands = []  # type: List[Command]

    def get_tag_name(self, new_version: str) -> str:
        return self.tag_template.format(new_version=new_version)

    def set_config(self, config: Config) -> None:
        self.tag_template = config.git_tag_template
        self.message_template = config.git_message_template

    def run_git(self, *args: str, verbose: bool = False) -> None:
        full_args = [self.repo_path] + list(args)
        return tbump.git.run_git(*full_args, verbose=verbose)

    def run_git_captured(self, *args: str, check: bool = True) -> Tuple[int, str]:
        full_args = [self.repo_path] + list(args)
        return tbump.git.run_git_captured(*full_args, check=check)

    def check_dirty(self) -> None:
        _, out = self.run_git_captured("status", "--porcelain")
        dirty = False
        for line in out.splitlines():
            # Ignore untracked files
            if not line.startswith("??"):
                dirty = True
        if dirty:
            raise DirtyRepository(git_status_output=out)

    def get_current_branch(self) -> str:
        cmd = ("rev-parse", "--abbrev-ref", "HEAD")
        _, out = self.run_git_captured(*cmd)
        if out == "HEAD":
            raise NotOnAnyBranch()
        return out

    def get_tracking_ref(self) -> str:
        branch_name = self.get_current_branch()
        rc, out = self.run_git_captured(
            "rev-parse", "--abbrev-ref", "--symbolic-full-name", "@{u}", check=False
        )
        if rc != 0:
            raise NoTrackedBranch(branch=branch_name)
        return out

    def check_ref_does_not_exists(self, tag_name: str) -> None:
        rc, _ = self.run_git_captured("rev-parse", tag_name, check=False)
        if rc == 0:
            raise RefAlreadyExists(ref=tag_name)

    def check_branch_state(self, new_version: str) -> None:
        self.get_current_branch()
        tag_name = self.get_tag_name(new_version)
        self.check_ref_does_not_exists(tag_name)

        tracking_ref = self.get_tracking_ref()
        self.remote_name, self.remote_branch = tracking_ref.split("/", maxsplit=1)

    def add_command(self, commands: List[Command], *cmd: str) -> None:
        command = Command(self.repo_path, list(cmd))
        commands.append(command)

    def get_commands(self, new_version: str) -> List[Command]:
        res = []  # type: List[Command]
        self.add_command(res, "add", "--update")
        commit_message = self.message_template.format(new_version=new_version)
        self.add_command(res, "commit", "--message", commit_message)
        tag_name = self.get_tag_name(new_version)
        tag_message = tag_name
        self.add_command(res, "tag", "--annotate", "--message", tag_message, tag_name)
        self.add_command(res, "push", self.remote_name, self.remote_branch)
        self.add_command(res, "push", self.remote_name, tag_name)
        return res



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tbump', 'tbump.test', 'tbump.test.data']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0',
 'cli-ui>=0.10.3,<0.11.0',
 'docopt>=0.6.2,<0.7.0',
 'path>=13.0,<14.0',
 'schema>=0.7.1,<0.8.0',
 'tomlkit>=0.5.8,<0.6.0']

entry_points = \
{'console_scripts': ['tbump = tbump.main:main']}

setup_kwargs = {
    'name': 'tbump',
    'version': '6.1.0',
    'description': 'Bump software releases',
    'long_description': '.. image::  https://tanker.io/images/github-logo.png\n   :target: #readme\n\n|\n\n.. image:: https://img.shields.io/pypi/v/tbump.svg\n  :target: https://pypi.org/project/tbump/\n\n.. image:: https://img.shields.io/github/license/TankerHQ/tbump.svg\n  :target: https://github.com/TankerHQ/tbump/blob/master/LICENSE\n\n.. image:: https://github.com/TankerHQ/tbump/workflows/tests/badge.svg\n   :target: https://github.com/TankerHQ/tbump/actions\n\n.. image:: https://github.com/TankerHQ/tbump/workflows/linters/badge.svg\n   :target: https://github.com/TankerHQ/tbump/actions\n\n.. image:: https://img.shields.io/codecov/c/github/TankerHQ/tbump.svg?label=Coverage\n   :target: https://codecov.io/gh/TankerHQ/tbump\n\n.. image:: https://img.shields.io/badge/deps%20scanning-pyup.io-green\n   :target: https://github.com/TankerHQ/tbump/workflows/safety/\n\n.. image:: https://img.shields.io/badge/code%20style-black-black.svg\n   :target: https://github.com/psf/black\n\n.. image:: https://img.shields.io/badge/mypy-checked-blue.svg\n   :target: https://mypy-lang.org\n\n\ntbump: bump software releases\n=============================\n\nInstallation\n------------\n\n\nThe recommended way to install ``tbump`` is to use `pipx <https://pipxproject.github.io/pipx/>`_\n\n* Make sure to have Python **3.5** or later installed.\n* Install ``pipx``\n* Run ``pipx install tbump``.\n\n``tbump`` is also available on ``pypi`` and can be installed with ``pip`` if you know what you are doing.\n\nScreenshot\n-----------\n\nHere\'s what a typical usage of ``tbump`` looks like:\n\n.. code-block:: console\n\n    $ tbump 5.0.5\n    :: Bumping from 5.0.4 to 5.0.5\n    => Would patch these files\n    - setup.py:14 version="5.0.4",\n    + setup.py:14 version="5.0.5",\n    - tbump.toml:2 current = "5.0.4"\n    + tbump.toml:2 current = "5.0.5"\n    => Would run these hooks before commit\n    * (1/2) $ ./test.sh\n    * (2/2) $ grep -q 5.0.5 Changelog.rst\n    => Would run these git commands\n     * git add --update\n     * git commit --message Bump to 5.0.5\n     * git tag --annotate --message v5.0.5 v5.0.5\n     * git push origin master\n     * git push origin v5.0.5\n    => Would run these hooks after push\n    * (1/1) $ ./publish.sh\n    :: Looking good? (y/N)\n    y\n    => Patching files\n    ...\n    => Running hooks before commit\n    ...\n    => Making bump commit and push matching tags\n    ...\n    => Running hooks after push\n    ...\n    Done ✓\n\n\n\nUsage\n------\n\nFirst, run ``tbump init``. This will create a ``tbump.toml`` file looking like this:\n\n.. code-block:: ini\n\n    [version]\n    current = "1.2.41"\n    regex = \'\'\'\n      (?P<major>\\d+)\n      \\.\n      (?P<minor>\\d+)\n      \\.\n      (?P<patch>\\d+)\n    \'\'\'\n\n    [git]\n    message_template = "Bump to {new_version}"\n    tag_template = "v{new_version}"\n\n    [[file]]\n    src = "setup.py"\n\n\n.. note::\n\n * The file uses `toml syntax <https://github.com/toml-lang/toml>`_.\n * Strings should be templated using curly brackets, to be used with Python\'s built-in ``.format()`` method.\n * Paths may contain unix-style `globs <https://docs.python.org/3/library/glob.html>`_, e.g. ``src = "a/**/script.?s"`` matches both ``a/b/script.js`` and ``a/b/c/script.ts``.\n * The version regular expression will be used in `verbose mode <https://docs.python.org/3/library/re.html#re.VERBOSE>`_ and can contain named groups (see below).\n\nThen run:\n\n.. code-block:: console\n\n    $ tbump 1.2.42\n\n``tbump`` will:\n\n* Replace the string ``1.2.41`` by ``1.2.42`` in every file listed in the\n  configuration\n\n* Make a commit based on the ``message_template``.\n\n* Make an **annotated** tag based on the ``tag_template``\n\n* Push the current branch and the tag.\n\nNote that by default, ``tbump`` will display all the changes and stop to ask if they are correct before performing any action, allowing you to abort and re-try the bump if something is not right.\nYou can use ``--non-interactive`` to disable this behavior.\n\nIf you only want to bump the files without performing any git actions or running the hook commands, use the ``--only-patch`` option.\n\nAdvanced configuration\n----------------------\n\nRestricting the lines that are replaced\n+++++++++++++++++++++++++++++++++++++++\n\n\nSometimes you want to make sure only the line matching a given pattern is replaced. For instance, with the following ``package.json``:\n\n.. code-block:: js\n\n    /* in package.json */\n    {\n       "name": "foo",\n       "version": "0.42",\n       "dependencies": {\n         "some-dep": "0.42",\n         "other-dep": "1.3",\n       }\n    }\n\nyou\'ll want to make sure that when you bump from ``0.42`` to ``0.43`` that the line containing ``some-dep`` does not change.\n\nIn this case, you can set a ``search`` option in the ``file`` section:\n\n.. code-block:: ini\n\n    # In tbump.toml\n\n    [[file]]\n    src = "package.json"\n    search = \'"version": "{current_version}"\'\n\nNote that the search string is actually a full regular expression, except for the ``{current_version}`` marker which is substituted as plain text.\n\n\nUsing a custom version template\n+++++++++++++++++++++++++++++++\n\nIf you are using a version schema like ``1.2.3-alpha-4``, you may want to expose a variable that only contains the "public" part of the version string. (``1.2.3`` in this case).\n\nTo do so, add a ``version_template`` option in the ``file`` section. The names used in the format string should match the group names in the regular expression.\n\n\n.. code-block:: js\n\n      /* in version.js */\n\n      export FULL_VERSION = \'1.2.3-alpha-4\';\n      export PUBLIC_VERSION = \'1.2.3\';\n\n.. code-block:: ini\n\n\n      [[file]]\n      src = "version.js"\n      version_template = "{major}.{minor}.{patch}"\n      search = "export PUBLIC_VERSION = \'{current_version}\'"\n\n      [[file]]\n      src = "version.js"\n      search = "export FULL_VERSION = \'{current_version}\'"\n\n\nRunning commands before commit\n++++++++++++++++++++++++++++++\n\nYou can specify a list of hooks to be run after the file have changed, but before the commit is made and pushed.\n\nThis is useful if some of the files under version control are generated through an external program.\n\nHere\'s an example:\n\n\n.. code-block:: ini\n\n    [[before_commit]]\n    name = "Check Changelog"\n    cmd = "grep -q {new_version} Changelog.rst"\n\n\nThe name is mandatory. The command will be executed via the shell, after the  ``{new_version}``  placeholder is replaced with the new version.\n\nAny hook that fails will interrupt the bump. You may want to run ``git reset --hard`` before trying again to undo the changes made in the files.\n\nRunning commands after push\n+++++++++++++++++++++++++++\n\nYou can specify a list of hooks to be run right after the tag has been pushed, using an `[[after_push]]` section.\n\nThis is useful if you need the command to run on a clean repository, without un-committed changes, for instance to publish ``rust`` packages:\n\n.. code-block:: ini\n\n    [[after_push]]\n    name = "Publish to crates.io"\n    cmd = "cargo publish"\n',
    'author': 'Dimitri Merejkowsky',
    'author_email': 'd.merej@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



import pytest

from path import Path
import tbump.file_bumper
from tbump.test.conftest import file_contains


def test_file_bumper_simple(test_repo: Path) -> None:
    bumper = tbump.file_bumper.FileBumper(test_repo)
    config = tbump.config.parse(test_repo / "tbump.toml")
    assert bumper.working_path == test_repo
    bumper.set_config(config)
    patches = bumper.get_patches(new_version="1.2.41-alpha-2")
    for patch in patches:
        patch.apply()

    assert file_contains(test_repo / "package.json", '"version": "1.2.41-alpha-2"')
    assert file_contains(test_repo / "package.json", '"other-dep": "1.2.41-alpha-1"')
    assert file_contains(test_repo / "pub.js", "PUBLIC_VERSION = '1.2.41'")
    assert file_contains(test_repo / "glob-one.c", 'version_one = "1.2.41-alpha-2"')
    assert file_contains(test_repo / "glob-two.v", 'version_two = "1.2.41-alpha-2"')


def test_patcher_preserve_endings(tmp_path: Path) -> None:
    foo_txt = tmp_path / "foo.txt"
    old_contents = b"line 1\r\nv=42\r\nline3\r\n"
    foo_txt.write_bytes(old_contents)
    patch = tbump.file_bumper.Patch(tmp_path, "foo.txt", 1, "v=42", "v=43")
    patch.apply()
    actual_contents = foo_txt.bytes()
    expected_contents = old_contents.replace(b"42", b"43")
    assert actual_contents == expected_contents


def test_file_bumper_preserve_endings(test_repo: Path) -> None:
    bumper = tbump.file_bumper.FileBumper(test_repo)
    config = tbump.config.parse(test_repo / "tbump.toml")
    package_json = test_repo / "package.json"

    # Make sure package.json contain CRLF line endings
    lines = package_json.lines(retain=False)
    package_json.write_text("\r\n".join(lines), linesep=None)

    bumper.set_config(config)
    patches = bumper.get_patches(new_version="1.2.41-alpha-2")
    for patch in patches:
        patch.apply()

    actual = package_json.bytes()
    assert b'version": "1.2.41-alpha-2",\r\n' in actual


def test_looking_for_empty_groups(tmp_path: Path) -> None:
    tbump_path = tmp_path / "tbump.toml"
    tbump_path.write_text(
        r"""
        [version]
        current = "1.2"
        regex = '''
            (?P<major>\d+)
            \.
            (?P<minor>\d+)
            (
              \.
              (?P<patch>\d+)
            )?
        '''

        [git]
        message_template = "Bump to {new_version}"
        tag_template = "v{new_version}"

        [[file]]
        src = "foo"
        version_template = "{major}.{minor}.{patch}"

        """
    )
    foo_path = tmp_path / "foo"
    foo_path.write_text(
        """
        version = "1.2"
        """
    )
    config = tbump.config.parse(tbump_path)
    bumper = tbump.file_bumper.FileBumper(tmp_path)
    bumper.set_config(config)
    with pytest.raises(tbump.file_bumper.BadSubstitution) as e:
        bumper.get_patches(new_version="1.3.1")
    assert e.value.src == "foo"
    assert e.value.groups == {"major": "1", "minor": "2", "patch": None}


def test_current_version_not_found(tmp_path: Path) -> None:
    tbump_path = tmp_path / "tbump.toml"
    tbump_path.write_text(
        r"""
        [version]
        current = "1.2.3"
        regex = ".*"

        [git]
        message_template = "Bump to {new_version}"
        tag_template = "v{new_version}"

        [[file]]
        src = "version.txt"
        """
    )
    version_txt_path = tmp_path / "version.txt"
    version_txt_path.write_text("nope")
    config = tbump.config.parse(tbump_path)

    bumper = tbump.file_bumper.FileBumper(tmp_path)
    bumper.set_config(config)
    with pytest.raises(tbump.file_bumper.CurrentVersionNotFound) as e:
        bumper.get_patches(new_version="1.3.1")
    assert e.value.src == "version.txt"


def test_replacing_with_empty_groups(tmp_path: Path) -> None:
    tbump_path = tmp_path / "tbump.toml"
    tbump_path.write_text(
        r"""
        [version]
        current = "1.2.3"
        regex = '''
            (?P<major>\d+)
            \.
            (?P<minor>\d+)
            (
              \.
              (?P<patch>\d+)
            )?
        '''

        [git]
        message_template = "Bump to {new_version}"
        tag_template = "v{new_version}"

        [[file]]
        src = "foo"
        version_template = "{major}.{minor}.{patch}"

        """
    )
    foo_path = tmp_path / "foo"
    foo_path.write_text(
        """
        version = "1.2.3"
        """
    )

    bumper = tbump.file_bumper.FileBumper(tmp_path)
    config = tbump.config.parse(tbump_path)
    bumper.set_config(config)
    with pytest.raises(tbump.file_bumper.BadSubstitution) as e:
        bumper.get_patches(new_version="1.3")
    assert e.value.groups == {"major": "1", "minor": "3", "patch": None}


def test_changing_same_file_twice(tmp_path: Path) -> None:
    tbump_path = tmp_path / "tbump.toml"
    tbump_path.write_text(
        r"""
        [version]
        current = "1.2.3"
        regex = '''
            (?P<major>\d+)
            \.
            (?P<minor>\d+)
            (
              \.
              (?P<patch>\d+)
            )?
        '''

        [git]
        message_template = "Bump to {new_version}"
        tag_template = "v{new_version}"

        [[file]]
        src = "foo.c"
        version_template = "{major}.{minor}"
        search = "PUBLIC_VERSION"

        [[file]]
        src = "foo.c"
        search = "FULL_VERSION"

        """
    )

    foo_c = tmp_path / "foo.c"
    foo_c.write_text(
        """
        #define FULL_VERSION "1.2.3"
        #define PUBLIC_VERSION "1.2"
        """
    )
    bumper = tbump.file_bumper.FileBumper(tmp_path)
    config = tbump.config.parse(tbump_path)
    bumper.set_config(config)
    patches = bumper.get_patches(new_version="1.3.0")
    for patch in patches:
        patch.do()

    assert file_contains(tmp_path / foo_c, '#define FULL_VERSION "1.3.0"')
    assert file_contains(tmp_path / foo_c, '#define PUBLIC_VERSION "1.3"')



from typing import List, Sequence

import cli_ui as ui

from tbump.action import Action
from tbump.git_bumper import GitBumper
from tbump.file_bumper import FileBumper
from tbump.hooks import HooksRunner

_ = List


class ActionGroup:
    def __init__(
        self,
        dry_run_desc: str,
        desc: str,
        actions: Sequence[Action],
        *,
        should_enumerate: bool = False
    ):
        self.should_enumerate = should_enumerate
        self.desc = desc
        self.dry_run_desc = dry_run_desc
        self.actions = actions

    def print_group(self, dry_run: bool = False) -> None:
        if not self.actions:
            return
        if dry_run:
            ui.info_2(self.dry_run_desc)
        else:
            ui.info_2(self.desc)
        for i, action in enumerate(self.actions):
            if self.should_enumerate:
                ui.info_count(i, len(self.actions), end="")
            action.print_self()

    def execute(self) -> None:
        for action in self.actions:
            action.do()


class Executor:
    def __init__(self, new_version: str, file_bumper: FileBumper):
        self.new_version = new_version
        self.work = []  # type: List[ActionGroup]

        patches = ActionGroup(
            "Would patch these files",
            "Patching files",
            file_bumper.get_patches(new_version),
        )
        self.work.append(patches)

    def add_git_and_hook_actions(
        self, new_version: str, git_bumper: GitBumper, hooks_runner: HooksRunner
    ) -> None:
        before_hooks = ActionGroup(
            "Would run these hooks before commit",
            "Running hooks before commit",
            hooks_runner.get_before_hooks(new_version),
            should_enumerate=True,
        )
        self.work.append(before_hooks)

        git_commands = ActionGroup(
            "Would run these git commands",
            "Making bump commit and push matching tag",
            git_bumper.get_commands(new_version),
        )
        self.work.append(git_commands)

        after_hooks = ActionGroup(
            "Would run these hooks after push",
            "Running hooks after push",
            hooks_runner.get_after_hooks(new_version),
            should_enumerate=True,
        )
        self.work.append(after_hooks)

    def print_self(self, *, dry_run: bool = False) -> None:
        for action_group in self.work:
            action_group.print_group(dry_run=dry_run)

    def run(self) -> None:
        for action_group in self.work:
            action_group.print_group(dry_run=False)
            action_group.execute()
